/**
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecManager singleton class methods definition
 */

#include "TeecManager.h"

#include <common/platform_exception.h>


namespace {

static uint32_t opIDCounter = 0;

} // namespace

namespace extension {
namespace libteec {

TeecManager& TeecManager::Instance() {
    static TeecManager manager;
    return manager;
}

TeecContextPtr TeecManager::GetContext(const std::string& contextName) {
    auto it = mContexts.find(contextName);
    if (it == mContexts.end()) {
        // no context - create new
        TeecContextPtr context = std::make_shared<TeecContext>(contextName);
        it = mContexts.insert(std::make_pair(contextName, context)).first;
    }

    return it->second;
}

uint32_t TeecManager::CreateOperation() {
    std::lock_guard<std::mutex> lock(mOperationListMutex);
    opIDCounter++;
    mOperations.emplace(opIDCounter, TEEC_Operation());
    return opIDCounter;
}

TEEC_Operation TeecManager::GetOperation(uint32_t id) const {
    std::lock_guard<std::mutex> lock(mOperationListMutex);
    auto it = mOperations.find(id);
    if (it == mOperations.end()) {
        throw common::InvalidValuesException("Operation " + std::to_string(id) + " not found");
    }
    return it->second;
}

void TeecManager::RemoveOperation(uint32_t id) {
    std::lock_guard<std::mutex> lock(mOperationListMutex);
    mOperations.erase(id);
}

} // namespace libteec
} // namespace extension
