/**
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecSession class methods definition
 */

#include "TeecSession.h"
#include "TeecTranslations.h"

#include "common/logger.h"
#include "common/platform_exception.h"


namespace {

const uint32_t UUID_UNIFIED_LENGTH = 32;

// assumes UUID already was unified by TeecContext
TEEC_UUID StringToUUID(const std::string& uuidstr) {
    if (uuidstr.size() != UUID_UNIFIED_LENGTH) {
        throw common::InvalidValuesException("Bad UUID provided: " + uuidstr);
    }

    TEEC_UUID uuid;
    uuid.timeLow = std::stoul(uuidstr.substr(0, 8), 0, 16);
    uuid.timeMid = static_cast<uint16_t>(std::stoul(uuidstr.substr(8, 4), 0, 16));
    uuid.timeHiAndVersion = static_cast<uint16_t>(std::stoul(uuidstr.substr(12, 4), 0, 16));

    uint32_t stringPos = 16;
    for (uint32_t i = 0; i < 8; ++i) {
        uuid.clockSeqAndNode[i] = static_cast<uint8_t>(std::stoul(uuidstr.substr(stringPos, 2), 0, 16));
        stringPos += 2;
    }

    return uuid;
}

} // namespace

namespace extension {
namespace libteec {

TeecSession::TeecSession(TEEC_Context* ctx, const std::string& uuidstr,
                        uint32_t connectionMethod, const void* connectionData,
                        TEEC_Operation* operation, uint32_t* returnOrigin) {
    TEEC_UUID uuid = StringToUUID(uuidstr);
    TEEC_Result result = TEEC_OpenSession(ctx, &mSession, &uuid, connectionMethod,
                                          connectionData, operation, returnOrigin);
    if (result != TEEC_SUCCESS) {
        throw common::InvalidValuesException("TEEC Session failed to open: " +
                                             translations::TeecResultToString(result));
    }
}

TeecSession::~TeecSession() {
    TEEC_CloseSession(&mSession);
}

void TeecSession::InvokeCommand(uint32_t cmd, TEEC_Operation* op, uint32_t* returnOrigin) {
    TEEC_Result result = TEEC_InvokeCommand(&mSession, cmd, op, returnOrigin);
    if (result != TEEC_SUCCESS) {
        throw common::InvalidValuesException("Failure while invoking command " +
                                             std::to_string(cmd) + ": " +
                                             translations::TeecResultToString(result));
    }
}

} // namespace libteec
} // namespace extension
