/**
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecSharedMemory class methods definition
 */

#include "TeecSharedMemory.h"
#include "TeecTranslations.h"

#include <string>
#include "common/platform_exception.h"


namespace extension {
namespace libteec {

TeecSharedMemory::TeecSharedMemory(TEEC_Context* ctx, size_t size, uint32_t flags) {
    mMemory.buffer = nullptr;
    mMemory.size = size;
    mMemory.flags = flags;
    TEEC_Result result = TEEC_AllocateSharedMemory(ctx, &mMemory);
    if (result != TEEC_SUCCESS) {
        throw common::InvalidValuesException("Failed to allocate shared memory: " +
                                             translations::TeecResultToString(result) +
                                             " (size " + std::to_string(size) +
                                             ", flags: " + std::to_string(flags));
    }
}

TeecSharedMemory::TeecSharedMemory(TEEC_Context* ctx, void* buffer, size_t size, uint32_t flags) {
    mMemory.buffer = buffer;
    mMemory.size = size;
    mMemory.flags = flags;
    TEEC_Result result = TEEC_RegisterSharedMemory(ctx, &mMemory);
    if (result != TEEC_SUCCESS) {
        throw common::InvalidValuesException("Failed to allocate shared memory: " +
                                             translations::TeecResultToString(result) +
                                             " (size " + std::to_string(size) +
                                             ", flags: " + std::to_string(flags));
    }
}

TeecSharedMemory::~TeecSharedMemory() {
    TEEC_ReleaseSharedMemory(&mMemory);
}

void TeecSharedMemory::SetData(const DataBuffer& data, size_t offset) {
    if (data.size() > (mMemory.size - offset)) {
        throw common::TypeMismatchException("Not enough memory to set data: " +
                                            std::to_string(data.size()) +
                                            " requested, available " +
                                            std::to_string(mMemory.size - offset) +
                                            " at offset " + std::to_string(offset));
    }

    char* sharedBuffer = reinterpret_cast<char*>(mMemory.buffer);
    memcpy(sharedBuffer + offset, data.data(), data.size());
}

void TeecSharedMemory::GetData(DataBuffer& data, size_t offset) const {
    if (data.size() > (mMemory.size - offset)) {
        throw common::TypeMismatchException("Not enough memory to get data: " +
                                            std::to_string(data.size()) +
                                            " requested, available " +
                                            std::to_string(mMemory.size - offset) +
                                            " at offset " + std::to_string(offset));
    }

    char* sharedBuffer = reinterpret_cast<char*>(mMemory.buffer);
    memcpy(data.data(), sharedBuffer + offset, data.size());
}

TEEC_SharedMemory* TeecSharedMemory::GetMemoryPointer() {
    return &mMemory;
}

} // namespace libteec
} // namespace extension
